/* 
Modified linker script org: mspgcc/msp430/lib/ldscripts/msp430x1612.x 

Realizes the following memory layout in flash & ram:



RAM:
====
Firmware data & bss
Userapp data & bss


FLASH: 									
======
Firmware Header					
Firmware Text

App Header
App Text

Bootloader
IVT
ivt

*/


OUTPUT_FORMAT("elf32-msp430","elf32-msp430","elf32-msp430")
OUTPUT_ARCH(msp:16)
MEMORY
{
  text   (rx)   	: ORIGIN = 0x2500,  LENGTH = 0xdae0
  data   (rwx)  	: ORIGIN = 0x1100, 	LENGTH = 0x1400
  vectors (rw)  	: ORIGIN = 0xffe0,      LENGTH = 0x20
  bootloader(rx)	: ORIGIN = 0x0c00,	LENGTH = 1K
  infomem(rx)		: ORIGIN = 0x1000,	LENGTH = 256
  infomemnobits(rx)	: ORIGIN = 0x1000,      LENGTH = 256
}


/* This is for ensurance that no firmware function calls something of the library functions
   which get linked behind or with the userapp.
*/
NOCROSSREFS(.fwtext .libtext) 


/* This is for ensurance that div and mod functions alway get linked in to firmware */
EXTERN(__umulsi3hw __udivmodhi4 __divmodhi4 __udivmodsi4)

/* You can declare initialized variables, but they never get assigned:
   int i = 25; // <-- Initialization never runs, so i will be uninitilized regardless of what you write.
   So to avoid this sort of bugs, it is forbidden using variables in the .fwdata and .data section 
NOCROSSREFS(.fwtext .data .fwdata)
NOCROSSREFS(.text .data .fwdata)
*/

SECTIONS
{

  /*****************************************************************************
   ***                                                                       ***
   *** Text section (goes all into flash memory)                             ***
   ***                                                                       ***
   *****************************************************************************/

  .fwtext :
  {
		PROVIDE (__fw_start = .);  
    *.o(.fw_header)
    . = ALIGN(2);  
    *.o(.fw_main)
    . = ALIGN(2);      
    *.o(.text)  
    . = ALIGN(2);
    
    /* Div and Mod are needed in firmware!! */
    
    *_mulsi3hw.o(.text.libgcc);
    . = ALIGN(2);
    *_udivmodhi4.o(.text.libgcc);
    . = ALIGN(2);    
    *_divmodhi4.o(.text.libgcc);
    . = ALIGN(2);
    *_udivmodsi4.o(.text.libgcc);
    
    
    . = ALIGN(512);                  /* ADDED for programming.c:Align to 512b boundary. */
    
		PROVIDE (__fw_end = .);  

  } > text


  /* Internal text space.  */
  .text :
  {
 		PROVIDE (__app_start = .);  
    *(.app_header)	             /* ADDED for programming.c: Include data about this image (start, end, date, device type and name) at the beginning */  
    . = ALIGN(2);

    *(.text)
    . = ALIGN(2);
    
    PROVIDE (__term_start = .) ; /* ADDED for terminal.c: Make start address of .commands available to the program. */
    *(.commands)                 /* ADDED for terminal.c: Include commands segment. */
    PROVIDE (__term_end = .) ;   /* ADDED for terminal.c: Make end address of .commands available to the program. */    
    . = ALIGN(2);		 /* ADDED for terminal.c: */
  }  > text

  .libtext : 
  {
    *(.text.*)
    *_sf.o(.text)
    
    . = ALIGN(2);  
  } > text
  
	PROVIDE (__app_end = .);  
  _etext = .;
  
  
  
  /**** Loader and reprogrammer */
  .loader 0xFD00: 
  {
		PROVIDE (__loader_start = .);  
    *programming.o(.loader) 					/* ADDED for programming.c: Reprogramming code at the very beginning. */ 
		PROVIDE (__loader_end = .);  
  } > text  
  
  /* 64 Bytes, 4 Bytes for each ISR */
  .isr 0xFFA0 : 
  {
    fw_*(.isr)
  } > text
  
  .vectors  :
  {
     PROVIDE (__vectors_start = .) ;
    *(.vectors*)
     _vectors_end = . ;
  }  > vectors  





  /*****************************************************************************
   ***                                                                       ***
   *** Data + bss section (goes all into RAM memory)                         ***
   ***                                                                       ***
   *****************************************************************************/  

  .fwdata : 
  {
    *.o(.ivt)
    . = ALIGN(2);
    *.o(.data)
    . = ALIGN(2);
  }  > data
  
  .fwbss :
  {
    *.o(.bss)
    *.o(COMMON)
  }  > data
  
  
  .data : 
  {
     PROVIDE (__data_start = .) ;
    . = ALIGN(2);
    *(.data)
    . = ALIGN(2);
    *(.gnu.linkonce.d*)
    . = ALIGN(2);
  }  > data
  
  .bss :
  {
     PROVIDE (__bss_start = .) ;
    *(.bss)
    *(COMMON)
     PROVIDE (__bss_end = .) ;
  }  > data


  /*****************************************************************************
   ***                                                                       ***
   *** Other                                                                 ***
   ***                                                                       ***
   *****************************************************************************/  
  
  

  /* Stabs for profiling information*/
  .profiler 0 : { *(.profiler) }
  /* Stabs debugging sections.  */
  .stab 0 : { *(.stab) }
  .stabstr 0 : { *(.stabstr) }
  .stab.excl 0 : { *(.stab.excl) }
  .stab.exclstr 0 : { *(.stab.exclstr) }
  .stab.index 0 : { *(.stab.index) }
  .stab.indexstr 0 : { *(.stab.indexstr) }
  .comment 0 : { *(.comment) }
  /* DWARF debug sections.
     Symbols in the DWARF debugging sections are relative to the beginning
     of the section so we begin them at 0.  */
  /* DWARF 1 */
  .debug          0 : { *(.debug) }
  .line           0 : { *(.line) }
  /* GNU DWARF 1 extensions */
  .debug_srcinfo  0 : { *(.debug_srcinfo) }
  .debug_sfnames  0 : { *(.debug_sfnames) }
  /* DWARF 1.1 and DWARF 2 */
  .debug_aranges  0 : { *(.debug_aranges) }
  .debug_pubnames 0 : { *(.debug_pubnames) }
  /* DWARF 2 */
  .debug_info     0 : { *(.debug_info) *(.gnu.linkonce.wi.*) }
  .debug_abbrev   0 : { *(.debug_abbrev) }
  .debug_line     0 : { *(.debug_line) }
  .debug_frame    0 : { *(.debug_frame) }
  .debug_str      0 : { *(.debug_str) }
  .debug_loc      0 : { *(.debug_loc) }
  .debug_macinfo  0 : { *(.debug_macinfo) }
  
  
  PROVIDE (__stack = 0x2500) ;
  PROVIDE (__data_start_rom = _etext) ;
  PROVIDE (__data_end_rom   = _etext + SIZEOF (.data)) ;
  PROVIDE (__noinit_start_rom = _etext + SIZEOF (.data)) ;
  PROVIDE (__noinit_end_rom = _etext + SIZEOF (.data) + SIZEOF (.noinit)) ;
  PROVIDE (__image_end = _etext + SIZEOF (.data) + SIZEOF (.noinit)) ; /* ADDED */
  PROVIDE (__subdevice_has_heap = 0) ;
}
