/*
Copyright 2006, Freie Universitaet Berlin. All rights reserved.

These sources were developed at the Freie Universitt Berlin, Computer
Systems and Telematics group.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:

- Redistributions of source code must retain the above copyright
notice, this list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright
notice, this list of conditions and the following disclaimer in the
documentation and/or other materials provided with the distribution.
 
- Neither the name of Freie Universitaet Berlin (FUB) nor the names of its
contributors may be used to endorse or promote products derived from
this software without specific prior written permission.

This software is provided by FUB and the contributors on an "as is"
basis, without any representations or warranties of any kind, express
or implied including, but not limited to, representations or
warranties of non-infringement, merchantability or fitness for a
particular purpose. In no event shall FUB or contributors be liable
for any direct, indirect, incidental, special, exemplary, or
consequential damages (including, but not limited to, procurement of
substitute goods or services; loss of use, data, or profits; or
business interruption) however caused and on any theory of liability,
whether in contract, strict liability, or tort (including negligence
or otherwise) arising in any way out of the use of this software, even
if advised of the possibility of such damage.

This implementation was developed by the CST group at the FUB.

For documentation and questions please use the web site
http://scatterweb.mi.fu-berlin.de and the mailinglist
scatterweb@lists.spline.inf.fu-berlin.de (subscription via the Website).
Berlin, 2006
*/

/** \file ScatterWeb.CC1020.h
 ** \ingroup System
 ** \brief The ScatterWeb.CC1020 consists all of the fuctions that constitute
 ** the chipcon architecture.
 ** <p>
 **  The software for the transceiver CC1020 from Chipcon features:
 **  - Functions to enable & disable the radio. Receive current is around 25 - 30 mA,
 **  transmit current from 60 mA up to over 150 mA (depends on the transmit power)
 **  and sleep current is in the range of uA.
 **  - Setting of transmit power (done in the config). Dont set a value higher than 3200,
 **  because even the transmit power increases the carrier signal gets distorted and
 **  will be worse.
 **  - Interrupt driven Rx&Tx radio MAC logic (sending of packets with CSMA and random 
 **  backoff, CRC-16 checksums, acknowledgement with automatic retransmission).
 ** 
 ** @author          Marco Hutta
 ** @version         1.0
 ** @since           1.0  
 **/

#ifndef ScatterWeb_CC1020
#define ScatterWeb_CC1020

#define CC1020_RSS 0x42
#define TX_CURRENT 0x87
#define RX_CURRENT 0x86

/**
 * This member supports the ScatterWeb infrastructure
 * and is not intended to be used directly from your code.
 */
UINT8 CC1020_paPower;

/**
 * This member supports the ScatterWeb infrastructure
 * and is not intended to be used directly from your code.
 */
void CC1020_init();

/**
 * This member supports the ScatterWeb infrastructure
 * and is not intended to be used directly from your code.
 */
char CC1020_calibrate();

/**
 * This member supports the ScatterWeb infrastructure
 * and is not intended to be used directly from your code.
 */
char CC1020_setupTX(char TXANALOG);

/**
 * This member supports the ScatterWeb infrastructure
 * and is not intended to be used directly from your code.
 */
char CC1020_setupRX(char RXANALOG);

/**
 * This member supports the ScatterWeb infrastructure
 * and is not intended to be used directly from your code.
 */
void CC1020_setupPD();

/**
 * This member supports the ScatterWeb infrastructure
 * and is not intended to be used directly from your code.
 */
void CC1020_wakeupTX(char TXANALOG);

/**
 * This member supports the ScatterWeb infrastructure
 * and is not intended to be used directly from your code.
 */
void CC1020_wakeupRX(char RXANALOG);

/**
 * This member supports the ScatterWeb infrastructure
 * and is not intended to be used directly from your code.
 */
UINT8 CC1020_readRegister(UINT8 addr);

/**
 * This member supports the ScatterWeb infrastructure
 * and is not intended to be used directly from your code.
 */
void CC1020_writeRegister(UINT8 addr, UINT8 adata);

/**
 * This member supports the ScatterWeb infrastructure
 * and is not intended to be used directly from your code.
 */
void CC1020_loadSettings();

/**
 * This member supports the ScatterWeb infrastructure
 * and is not intended to be used directly from your code.
 */
void CC1020_writeSettings();

/**
 * This member supports the ScatterWeb infrastructure
 * and is not intended to be used directly from your code.
 */ 
void CC1020_reset();

#endif
