/*
Copyright 2006, Freie Universitaet Berlin. All rights reserved.

These sources were developed at the Freie Universitt Berlin, Computer
Systems and Telematics group.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:

- Redistributions of source code must retain the above copyright
notice, this list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright
notice, this list of conditions and the following disclaimer in the
documentation and/or other materials provided with the distribution.
 
- Neither the name of Freie Universitaet Berlin (FUB) nor the names of its
contributors may be used to endorse or promote products derived from
this software without specific prior written permission.

This software is provided by FUB and the contributors on an "as is"
basis, without any representations or warranties of any kind, express
or implied including, but not limited to, representations or
warranties of non-infringement, merchantability or fitness for a
particular purpose. In no event shall FUB or contributors be liable
for any direct, indirect, incidental, special, exemplary, or
consequential damages (including, but not limited to, procurement of
substitute goods or services; loss of use, data, or profits; or
business interruption) however caused and on any theory of liability,
whether in contract, strict liability, or tort (including negligence
or otherwise) arising in any way out of the use of this software, even
if advised of the possibility of such damage.

This implementation was developed by the CST group at the FUB.

For documentation and questions please use the web site
http://scatterweb.mi.fu-berlin.de and the mailinglist
scatterweb@lists.spline.inf.fu-berlin.de (subscription via the Website).
Berlin, 2006
*/

/** \file ScatterWeb.Comm.h
 ** \ingroup System
 ** \brief The ScatterWeb.Comm consists all of the fuctions that constitute
 ** the serial architecture.
 ** 
 ** @author          Marco Hutta
 ** @version         1.0
 ** @since           1.0  
 **/

#ifndef ScatterWeb_Comm
#define ScatterWeb_Comm

/**
 * Define that sets the baudrate to 2400 Bit/s
 *
 * @see           #Comm_init(UINT8 bps)
 * @since         1.0
 */
#define BPS_2400       (0x01)

/**
 * Define that sets the baudrate to 4800 Bit/s
 *
 * @see           #Comm_init(UINT8 bps)
 * @since         1.0
 */
#define BPS_4800       (0x02)

/**
 * Define that sets the baudrate to 960000 Bit/s
 *
 * @see           #Comm_init(UINT8 bps)
 * @since         1.0
 */
#define BPS_9600       (0x03)

/**
 * Define that sets the baudrate to 14400 Bit/s
 *
 * @see           #Comm_init(UINT8 bps)
 * @since         1.0
 */
#define BPS_14400      (0x04)

/**
 * Define that sets the baudrate to 19200 Bit/s
 *
 * @see           #Comm_init(UINT8 bps)
 * @since         1.0
 */
#define BPS_19200      (0x05)

/**
 * Define that sets the baudrate to 38400 Bit/s
 *
 * @see           #Comm_init(UINT8 bps)
 * @since         1.0
 */
#define BPS_38400      (0x06)

/**
 * Define that sets the baudrate to 57600 Bit/s
 *
 * @see           #Comm_init(UINT8 bps)
 * @since         1.0
 */
#define BPS_57600      (0x07)

/**
 * Define that sets the baudrate to 1115200 Bit/s
 *
 * @see           #Comm_init(UINT8 bps)
 * @since         1.0
 */
#define BPS_115200     (0x08)

/**
 * Value of the log level
 *
 * @see           #Comm_log(UINT8, const char*, ...)
 * @since         1.0
 */
extern UINT8 Comm_logLevel;        

/**
 * Initializes the serial
 *
 * @param bps     the bps to be set 
 * @return        <code>True</code> always.
 * @since         1.0
 */
bool Comm_init(UINT8 bps);

/**
 * Sends bytes of data
 *
 * @param fmt     function
 * @param ...     list of parameters
 * @return        <code>True</code> always.
 * @since         1.0
 */
bool Comm_print(const char *fmt, ...);

/**
 * Sends bytes of log data
 *
 * @param level   log level
 * @param fmt     function
 * @param ...     list of parameters 
 * @return        <code>True</code> always.
 * @since         1.0
 */
bool Comm_log(UINT8 level, const char* fmt, ...);

/**
 * This member supports the ScatterWeb infrastructure
 * and is not intended to be used directly from your code.
 */
bool Comm_clear();

#endif
